==> ocp-0.1.0.tar.bz2: ocp-0.1.0/devw/dwnone_asminc.c <==
/* OpenCP Module Player
 * copyright (c) '94-'04 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
 *
 * auxiliary assembler routines for no sound wavetable device
 *
 * revision history: (please note changes here)
 *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
 *    -first release
 *  -ss04????   Stian Skjelstad <stian@nixia.no>
 *    -assembler rewritten to gcc
 *  -ss040908   Stian Skjelstad <stian@nixia.no>
 *    -minor changes to make gcc able to compile, even with VERY high optimize level
 */

#define MCP_PLAYING 1
#define MCP_LOOPED 4
#define MCP_PINGPONGLOOP 8


/* But ugly, and no usage of the fpos ... pure 32 bit for now */

#ifdef I386_ASM

static void nonePlayChannel(unsigned long len, struct channel *ch)
{
	int d0;
	unsigned char inloop;
	__asm__ __volatile__
	(
		"1:\n"   /* bigloop */
		" movl %2, %%ecx\n"
		" movl %4(%%edi), %%ebx\n"
		" movl %5(%%edi), %%edx\n"
		" movw %6(%%edi), %%si\n"
		" movb $0, %3\n"
		" cmpl $0, %%ebx\n"
		" je 5f\n" /* playecx */
		" jg 2f\n" /* forward */
		" negl %%ebx\n"
		" movl %%edx, %%eax\n"
		" testb $MCP_LOOPED, %7(%%edi)\n"
		" jz 4f\n" /* maxplaylen */
		" cmpl %8(%%edi), %%edx\n"
		" jb 4f\n" /* maxplaylen */
		" sub %8(%%edi), %%eax\n"
		" movb $1, %3\n"
		" jmp 4f\n" /* maxplaylen */
		"2:\n" /* forward */
		" movl %9(%%edi), %%eax\n"
		" negw %%si\n"
		" sbbl %%edx, %%eax\n"
		" testb $MCP_LOOPED, %7(%%edi)\n"
		" jz 4f\n" /* maxplaylen */
		" cmpl %%edx, %10(%%edi)\n"
		" jae 4f\n" /* maxplaylen */
		" subl %9(%%edi), %%eax\n"
		" addl %10(%%edi), %%eax\n"
		" movb $1, %3\n"

		"4:\n" /* maxplaylen */
		" xorl %%edx, %%edx\n"
		" shld $16, %%eax, %%edx\n"
		" shll $16, %%esi\n"
		" shld $16, %%esi, %%eax\n"
		" addl %%edx, %%eax\n"
		" adcl $0, %%eax\n"
		" subl $1, %%eax\n"
		" sbbl $0, %%edx\n"
		" cmpl %%ebx, %%edx\n"
		" jae 5f\n" /* playecx */
		" divl %%ebx\n"
		" cmpl %%eax, %%ecx\n"
		" jb 5f\n" /* playecx */
		"  movl %%eax, %%ecx\n"
		"  cmpb $0, %3\n"
		"  jnz 5f\n" /* playecx */
		"     andb $0xfe, %7(%%edi)\n" /* 0xfe is NOT MCP_PLAYING */
		"     movl %%ecx,%2\n"

		"5:\n"
		" subl %%ecx, %2\n"
		" movl %4(%%edi), %%eax\n"
		" imul %%ecx\n"
		" shld $16, %%eax, %%edx\n"
		" addw %%ax, %6(%%edi)\n"
		" adcl %%edx, %5(%%edi)\n"
		" movl %5(%%edi), %%eax\n"

		" cmpl $0, %3\n"
		" jz 9f\n" /* exit */

		" cmpl $0, %4(%%edi)\n"
		" jge 3f\n" /* forward2 */
		"   cmpl %8(%%edi), %%eax\n"
		"   jge 9f\n" /* exit */
		"   testb $MCP_PINGPONGLOOP,%7(%%edi)\n"
		"   jnz 6f\n" /* pong */
		"     addl %11(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"6:\n" /* pong */
		"     negl %4(%%edi)\n"
		"     negl %4(%%edi)\n"
		"     adcl $0, %%eax\n"
		"     negl %%eax\n"
		"     add  %8(%%edi), %%eax\n"
		"     add  %8(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"3:\n"
		"   cmpl %10(%%edi), %%eax\n"
		"   jb 9f\n" /* exit */
		"   testb $MCP_PINGPONGLOOP, %7(%%edi)\n"
		"   jnz 7f\n" /* ping */
		"     subl %11(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"7:\n" /* ping */
		"     negl %4(%%edi)\n"
		"     negw %6(%%edi)\n"
		"     adcl $0, %%eax\n"
		"     negl %%eax\n"
		"     addl %10(%%edi), %%eax\n"
		"     addl %10(%%edi), %%eax\n"

		"8:\n" /* loopiflen */
		" movl %%eax, %5(%%edi)\n"
		" cmpl $0, %0\n"
		" jne 1b\n"

		"9:\n"
		: "=&D"(d0)
		: "0" (ch),
       		  "m" (len),
		  "m" (inloop),
		  "m" (((struct channel *)0)->step),
		  "m" (((struct channel *)0)->pos),
		  "m" (((struct channel *)0)->fpos),
		  "m" (((struct channel *)0)->status),
		  "m" (((struct channel *)0)->loopstart),
		  "m" (((struct channel *)0)->length),
		  "m" (((struct channel *)0)->loopend),
		  "m" (((struct channel *)0)->replen)
		: "memory", "eax", "ebx", "ecx", "edx", "esi", "ebp"/*just to flag it off*/
	);
}

#else

broken void nonePlayChannel(unsigned long len, struct channel *ch)
{
	if (!(ch->status&MCP_PLAYING))
		return;
	if ((!ch->step)||(!len))
		return; /* if no length, or pitch is zero.. we don't need to simulate shit*/

	len=imuldiv(len, ch->step, 0x10000);
	
	while(len)
	{
		if (ch->step>0)
		{
			int landingzone=ch->pos+len+1;
			if (ch->status&MCP_LOOPED)
			{
				if (landingzone>=ch->loopend)
				{
					ch->pos=0;
					ch->fpos=0;
					ch->status^=MCP_PLAYING;
					break;
				} else {
					ch->pos+=len;
					break;
				}	
			} else {
				int landingzone=ch->pos+len-1;
				if (landingzone>=ch->loopend)
				{
					if (ch->status&MCP_PINGPONGLOOP)
					{
						ch->step=0-ch->step;
						ch->pos=ch->loopend;
					} else {
						ch->pos=ch->loopstart;
					}
					len=landingzone-ch->loopend;
				} else {
					ch->pos=landingzone;
					break;
				}	
			}
		} else {
			int landingzone=ch->pos-len-1;
			if (landingzone<=ch->loopstart)
			{
				ch->step=0-ch->step;
				ch->pos=ch->loopstart;
				len=landingzone-ch->loopend;
			} else {
				ch->pos=landingzone;
				break;
			}	
		}
	}
}

#endif

==> ocp-0.1.3.tar.bz2: ocp-0.1.3/devw/dwnone_asminc.c <==
/* OpenCP Module Player
 * copyright (c) '94-'04 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
 *
 * auxiliary assembler routines for no sound wavetable device
 *
 * revision history: (please note changes here)
 *  -nb980510   Niklas Beisert <nbeisert@physik.tu-muenchen.de>
 *    -first release
 *  -ss04????   Stian Skjelstad <stian@nixia.no>
 *    -assembler rewritten to gcc
 *  -ss040908   Stian Skjelstad <stian@nixia.no>
 *    -minor changes to make gcc able to compile, even with VERY high optimize level
 *  -doj040914  Dirk Jagdmann  <doj@cubic.org>
 *    -push/pop %ebp, and don't flag is as dirty in nonePlayChannel
 */

#define MCP_PLAYING 1
#define MCP_LOOPED 4
#define MCP_PINGPONGLOOP 8


/* Butt ugly, and no usage of the fpos ... pure 32 bit for now */

#ifdef I386_ASM

static void nonePlayChannel(unsigned long len, struct channel *ch)
{
	int d0;
	unsigned char inloop;
	__asm__ __volatile__
	(
		"pushl %%ebp\n"
		"1:\n"   /* bigloop */
		" movl %2, %%ecx\n"
		" movl %4(%%edi), %%ebx\n"
		" movl %5(%%edi), %%edx\n"
		" movw %6(%%edi), %%si\n"
		" movb $0, %3\n"
		" cmpl $0, %%ebx\n"
		" je 5f\n" /* playecx */
		" jg 2f\n" /* forward */
		" negl %%ebx\n"
		" movl %%edx, %%eax\n"
		" testb $MCP_LOOPED, %7(%%edi)\n"
		" jz 4f\n" /* maxplaylen */
		" cmpl %8(%%edi), %%edx\n"
		" jb 4f\n" /* maxplaylen */
		" sub %8(%%edi), %%eax\n"
		" movb $1, %3\n"
		" jmp 4f\n" /* maxplaylen */
		"2:\n" /* forward */
		" movl %9(%%edi), %%eax\n"
		" negw %%si\n"
		" sbbl %%edx, %%eax\n"
		" testb $MCP_LOOPED, %7(%%edi)\n"
		" jz 4f\n" /* maxplaylen */
		" cmpl %%edx, %10(%%edi)\n"
		" jae 4f\n" /* maxplaylen */
		" subl %9(%%edi), %%eax\n"
		" addl %10(%%edi), %%eax\n"
		" movb $1, %3\n"

		"4:\n" /* maxplaylen */
		" xorl %%edx, %%edx\n"
		" shld $16, %%eax, %%edx\n"
		" shll $16, %%esi\n"
		" shld $16, %%esi, %%eax\n"
		" addl %%edx, %%eax\n"
		" adcl $0, %%eax\n"
		" subl $1, %%eax\n"
		" sbbl $0, %%edx\n"
		" cmpl %%ebx, %%edx\n"
		" jae 5f\n" /* playecx */
		" divl %%ebx\n"
		" cmpl %%eax, %%ecx\n"
		" jb 5f\n" /* playecx */
		"  movl %%eax, %%ecx\n"
		"  cmpb $0, %3\n"
		"  jnz 5f\n" /* playecx */
		"     andb $0xfe, %7(%%edi)\n" /* 0xfe is NOT MCP_PLAYING */
		"     movl %%ecx,%2\n"

		"5:\n"
		" subl %%ecx, %2\n"
		" movl %4(%%edi), %%eax\n"
		" imul %%ecx\n"
		" shld $16, %%eax, %%edx\n"
		" addw %%ax, %6(%%edi)\n"
		" adcl %%edx, %5(%%edi)\n"
		" movl %5(%%edi), %%eax\n"

		" cmpl $0, %3\n"
		" jz 9f\n" /* exit */

		" cmpl $0, %4(%%edi)\n"
		" jge 3f\n" /* forward2 */
		"   cmpl %8(%%edi), %%eax\n"
		"   jge 9f\n" /* exit */
		"   testb $MCP_PINGPONGLOOP,%7(%%edi)\n"
		"   jnz 6f\n" /* pong */
		"     addl %11(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"6:\n" /* pong */
		"     negl %4(%%edi)\n"
		"     negl %4(%%edi)\n"
		"     adcl $0, %%eax\n"
		"     negl %%eax\n"
		"     add  %8(%%edi), %%eax\n"
		"     add  %8(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"3:\n"
		"   cmpl %10(%%edi), %%eax\n"
		"   jb 9f\n" /* exit */
		"   testb $MCP_PINGPONGLOOP, %7(%%edi)\n"
		"   jnz 7f\n" /* ping */
		"     subl %11(%%edi), %%eax\n"
		"     jmp 8f\n" /* loopiflen */
		"7:\n" /* ping */
		"     negl %4(%%edi)\n"
		"     negw %6(%%edi)\n"
		"     adcl $0, %%eax\n"
		"     negl %%eax\n"
		"     addl %10(%%edi), %%eax\n"
		"     addl %10(%%edi), %%eax\n"

		"8:\n" /* loopiflen */
		" movl %%eax, %5(%%edi)\n"
		" cmpl $0, %0\n"
		" jne 1b\n"

		"9:\n"
		" popl %%ebp\n"
		: "=&D"(d0)
		: "0" (ch),
       		  "m" (len),
		  "m" (inloop),
		  "m" (((struct channel *)0)->step),
		  "m" (((struct channel *)0)->pos),
		  "m" (((struct channel *)0)->fpos),
		  "m" (((struct channel *)0)->status),
		  "m" (((struct channel *)0)->loopstart),
		  "m" (((struct channel *)0)->length),
		  "m" (((struct channel *)0)->loopend),
		  "m" (((struct channel *)0)->replen)
		: "memory", "eax", "ebx", "ecx", "edx", "esi" /*just to flag it off*/
	);
}

#else

broken void nonePlayChannel(unsigned long len, struct channel *ch)
{
	if (!(ch->status&MCP_PLAYING))
		return;
	if ((!ch->step)||(!len))
		return; /* if no length, or pitch is zero.. we don't need to simulate shit*/

	len=imuldiv(len, ch->step, 0x10000);
	
	while(len)
	{
		if (ch->step>0)
		{
			int landingzone=ch->pos+len+1;
			if (ch->status&MCP_LOOPED)
			{
				if (landingzone>=ch->loopend)
				{
					ch->pos=0;
					ch->fpos=0;
					ch->status^=MCP_PLAYING;
					break;
				} else {
					ch->pos+=len;
					break;
				}	
			} else {
				int landingzone=ch->pos+len-1;
				if (landingzone>=ch->loopend)
				{
					if (ch->status&MCP_PINGPONGLOOP)
					{
						ch->step=0-ch->step;
						ch->pos=ch->loopend;
					} else {
						ch->pos=ch->loopstart;
					}
					len=landingzone-ch->loopend;
				} else {
					ch->pos=landingzone;
					break;
				}	
			}
		} else {
			int landingzone=ch->pos-len-1;
			if (landingzone<=ch->loopstart)
			{
				ch->step=0-ch->step;
				ch->pos=ch->loopstart;
				len=landingzone-ch->loopend;
			} else {
				ch->pos=landingzone;
				break;
			}	
		}
	}
}

#endif
